@@ -34,11 +34,9 @@
 #include "xf86.h"
 #include "xf86_OSproc.h"
 #include "xf86Pci.h"
-#include "xf86PciInfo.h"
 #include "xf86fbman.h"
 #include "xf86cmap.h"
 #include "compiler.h"
-#include "xaa.h"
 #include "mipointer.h"
 #include "micmap.h"
 #include "mibstore.h"
@@ -49,6 +47,12 @@
 #include "xf86Resources.h"
 #endif
 
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 12
+#define PIOOFFSET hwp->PIOOffset
+#else
+#define PIOOFFSET 0
+#endif
+
 #include <string.h>
 
 /*
@@ -58,27 +62,26 @@ static const OptionInfoRec * ARKAvailableOptions(int chipid, int busid);
 static void ARKIdentify(int flags);
 static Bool ARKProbe(DriverPtr drv, int flags);
 static Bool ARKPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool ARKEnterVT(int scrnIndex, int flags);
-static void ARKLeaveVT(int scrnIndex, int flags);
+static Bool ARKEnterVT(VT_FUNC_ARGS_DECL);
+static void ARKLeaveVT(VT_FUNC_ARGS_DECL);
 static void ARKSave(ScrnInfoPtr pScrn);
-static Bool ARKScreenInit(int scrnIndex, ScreenPtr pScreen, int argc,
-			  char **argv);
+static Bool ARKScreenInit(SCREEN_INIT_ARGS_DECL);
 static Bool ARKMapMem(ScrnInfoPtr pScrn);
 static void ARKUnmapMem(ScrnInfoPtr pScrn);
 static Bool ARKModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
-static void ARKAdjustFrame(int scrnIndex, int x, int y, int flags);
-Bool ARKSwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
-Bool ARKCloseScreen(int scrnIndex, ScreenPtr pScreen);
+static void ARKAdjustFrame(ADJUST_FRAME_ARGS_DECL);
+Bool ARKSwitchMode(SWITCH_MODE_ARGS_DECL);
+Bool ARKCloseScreen(CLOSE_SCREEN_ARGS_DECL);
 Bool ARKSaveScreen(ScreenPtr pScreen, int mode);
-static void ARKFreeScreen(int scrnIndex, int flags);
+static void ARKFreeScreen(FREE_SCREEN_ARGS_DECL);
 static void ARKLoadPalette(ScrnInfoPtr pScrn, int numColors,
 			   int *indicies, LOCO *colors,
 			   VisualPtr pVisual);
 static void ARKWriteMode(ScrnInfoPtr pScrn, vgaRegPtr pVga, ARKRegPtr new);
 
 /* helpers */
-static unsigned char get_daccomm(IOADDRESS);
-static unsigned char set_daccom(IOADDRESS, unsigned char comm);
+static unsigned char get_daccomm(unsigned long);
+static unsigned char set_daccom(unsigned long, unsigned char comm);
 
 
 _X_EXPORT DriverRec ARK =
@@ -165,10 +168,7 @@ static Bool ARKGetRec(ScrnInfoPtr pScrn)
 
 static void ARKFreeRec(ScrnInfoPtr pScrn)
 {
-	if (!pScrn->driverPrivate)
-		return;
-
-	xfree(pScrn->driverPrivate);
+	free(pScrn->driverPrivate);
 	pScrn->driverPrivate = NULL;
 }
 
@@ -202,7 +202,7 @@ static Bool ARKProbe(DriverPtr drv, int flags)
 					devSections, numDevSections, drv,
 					&usedChips);
 
-	xfree(devSections);
+	free(devSections);
 
 	if (numUsed <= 0)
 		return FALSE;
@@ -229,7 +229,7 @@ static Bool ARKProbe(DriverPtr drv, int flags)
 		foundScreen = TRUE;
 	}
 
-	xfree(usedChips);
+	free(usedChips);
 
 	return foundScreen;
 }
@@ -256,6 +256,7 @@ static Bool ARKPreInit(ScrnInfoPtr pScrn, int flags)
 		return FALSE;
 
 	hwp = VGAHWPTR(pScrn);
+	vgaHWSetStdFuncs(hwp);
 	vgaHWGetIOBase(hwp);
 
 	pScrn->monitor = pScrn->confScreen->monitor;
@@ -299,7 +300,7 @@ static Bool ARKPreInit(ScrnInfoPtr pScrn, int flags)
 	pARK = ARKPTR(pScrn);
 
 	xf86CollectOptions(pScrn, NULL);
-	if (!(pARK->Options = xalloc(sizeof(ARKOptions))))
+	if (!(pARK->Options = malloc(sizeof(ARKOptions))))
 		return FALSE;
 	memcpy(pARK->Options, ARKOptions, sizeof(ARKOptions));
 	xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, pARK->Options);
@@ -348,7 +349,7 @@ static Bool ARKPreInit(ScrnInfoPtr pScrn, int flags)
 		pARK->ChipRev = pARK->PciInfo->revision;
 #endif
 	}
-	xfree(pEnt);
+	free(pEnt);
 
 	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Chipset: \"%s\"\n", pScrn->chipset);
 
@@ -358,19 +359,19 @@ static Bool ARKPreInit(ScrnInfoPtr pScrn, int flags)
 #endif
 
 	/* unlock CRTC[0-7] */
-	outb(hwp->PIOOffset + hwp->IOBase + 4, 0x11);
-	tmp = inb(hwp->PIOOffset + hwp->IOBase + 5);
-	outb(hwp->PIOOffset + hwp->IOBase + 5, tmp & 0x7f);
-	modinx(hwp->PIOOffset + 0x3c4, 0x1d, 0x01, 0x01);
+	outb(PIOOFFSET + hwp->IOBase + 4, 0x11);
+	tmp = inb(PIOOFFSET + hwp->IOBase + 5);
+	outb(PIOOFFSET + hwp->IOBase + 5, tmp & 0x7f);
+	modinx(PIOOFFSET + 0x3c4, 0x1d, 0x01, 0x01);
 
-	/* use membase's later on ??? */
-	pARK->FBAddress = (rdinx(hwp->PIOOffset + 0x3c4, 0x13) << 16) +
-			  (rdinx(hwp->PIOOffset + 0x3c4, 0x14) << 24);
-
-	pScrn->memPhysBase = pARK->FBAddress;
+#ifndef XSERVER_LIBPCIACCESS
+	pScrn->memPhysBase = pARK->PciInfo->memBase[0];
+#else
+	pScrn->memPhysBase = pARK->PciInfo->regions[0].base_addr;
+#endif
 
 	xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "Framebuffer @ 0x%lx\n",
-		   (unsigned long)pARK->FBAddress);
+		   (unsigned long)pScrn->memPhysBase);
 
 	if (!xf86SetGamma(pScrn, gzeros))
 		return FALSE;
@@ -378,7 +379,7 @@ static Bool ARKPreInit(ScrnInfoPtr pScrn, int flags)
 	if (!pScrn->videoRam) {
 		unsigned char sr10;
 
-		sr10 = rdinx(hwp->PIOOffset + 0x3c4, 0x10);
+		sr10 = rdinx(PIOOFFSET + 0x3c4, 0x10);
 		if (pARK->Chipset == PCI_CHIP_1000PV) {
 			if ((sr10 & 0x40) == 0)
 				pScrn->videoRam = 1024;
@@ -403,9 +404,9 @@ static Bool ARKPreInit(ScrnInfoPtr pScrn, int flags)
 	{
 		int man_id, dev_id;
 
-		inb(hwp->PIOOffset + 0x3c6);		/* skip cmd register */
-		man_id = inb(hwp->PIOOffset + 0x3c6);	/* manufacturer id */
-		dev_id = inb(hwp->PIOOffset + 0x3c6);	/* device id */
+		inb(PIOOFFSET + 0x3c6);		/* skip cmd register */
+		man_id = inb(PIOOFFSET + 0x3c6);	/* manufacturer id */
+		dev_id = inb(PIOOFFSET + 0x3c6);	/* device id */
 		if (man_id == 0x84 && dev_id == 0x98) {
 			pARK->ramdac = ZOOMDAC;
 			pARK->dac_width = 16;
@@ -465,18 +466,18 @@ static Bool ARKPreInit(ScrnInfoPtr pScrn, int flags)
 
 	if (!pARK->NoAccel) {
 		if (!xf86LoadSubModule(pScrn, "xaa")) {
-			ARKFreeRec(pScrn);
-			return FALSE;
+			xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
+				   "XAA not available\n");
+			pARK->NoAccel = 1;
 		}
 	}
 
 	return TRUE;
 }
 
-static Bool ARKScreenInit(int scrnIndex, ScreenPtr pScreen, int argc,
-			  char **argv)
+static Bool ARKScreenInit(SCREEN_INIT_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	ARKPtr pARK = ARKPTR(pScrn);
 
 	pScrn->fbOffset = 0;
@@ -495,7 +496,7 @@ static Bool ARKScreenInit(int scrnIndex, ScreenPtr pScreen, int argc,
 
 	ARKSaveScreen(pScreen, SCREEN_SAVER_ON);
 
-	pScrn->AdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+	pScrn->AdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
 
 	miClearVisualTypes();
 	if (pScrn->bitsPerPixel > 8) {
@@ -542,13 +543,13 @@ static Bool ARKScreenInit(int scrnIndex, ScreenPtr pScreen, int argc,
 
 	if (!pARK->NoAccel) {
 		if (ARKAccelInit(pScreen)) {
-			xf86DrvMsg(scrnIndex, X_INFO, "Acceleration enabled\n");
+			xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Acceleration enabled\n");
 		} else {
-			xf86DrvMsg(scrnIndex, X_ERROR, "Acceleration initialization failed\n");
-			xf86DrvMsg(scrnIndex, X_INFO, "Acceleration disabled\n");
+			xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Acceleration initialization failed\n");
+			xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Acceleration disabled\n");
 		}
 	} else {
-			xf86DrvMsg(scrnIndex, X_INFO, "Acceleration disabled\n");
+			xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Acceleration disabled\n");
 	}
 
 	miDCInitialize(pScreen, xf86GetPointerScreenFuncs());
@@ -576,8 +577,8 @@ static void ARKSave(ScrnInfoPtr pScrn)
 	ARKPtr pARK = ARKPTR(pScrn);
 	ARKRegPtr save = &pARK->SavedRegs;
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
-	IOADDRESS isaIOBase = hwp->PIOOffset;
-	IOADDRESS vgaIOBase = isaIOBase + hwp->IOBase;
+	unsigned long isaIOBase = PIOOFFSET;
+	unsigned long vgaIOBase = isaIOBase + hwp->IOBase;
 
 	vgaHWUnlock(hwp);
 	vgaHWSave(pScrn, &hwp->SavedReg, VGA_SR_ALL);
@@ -641,8 +642,8 @@ static Bool ARKModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	int multiplexing, dac16, modepitch;
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
 	vgaRegPtr pVga = &hwp->ModeReg;
-	IOADDRESS isaIOBase = hwp->PIOOffset;
-	IOADDRESS vgaIOBase = isaIOBase + hwp->IOBase;
+	unsigned long isaIOBase = PIOOFFSET;
+	unsigned long vgaIOBase = isaIOBase + hwp->IOBase;
 	unsigned char tmp;
 	int offset;
 
@@ -748,8 +749,13 @@ static Bool ARKModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	new->sr10 = rdinx(isaIOBase + 0x3c4, 0x10) & ~0x1f;
 	new->sr10 |= 0x1f;
 
-	new->sr13 = pARK->FBAddress >> 16;
-	new->sr14 = pARK->FBAddress >> 24;
+#ifndef XSERVER_LIBPCIACCESS
+	new->sr13 = pARK->PciInfo->memBase[0] >> 16;
+	new->sr14 = pARK->PciInfo->memBase[0] >> 24;
+#else
+	new->sr13 = pARK->PciInfo->regions[0].base_addr >> 16;
+	new->sr14 = pARK->PciInfo->regions[0].base_addr >> 24;
+#endif
 
 	new->sr12 = rdinx(isaIOBase + 0x3c4, 0x12) & ~0x03;
 	switch (pScrn->videoRam) {
@@ -881,12 +887,12 @@ static Bool ARKModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 }
 
 
-static void ARKAdjustFrame(int scrnIndex, int x, int y, int flags)
+static void ARKAdjustFrame(ADJUST_FRAME_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 	ARKPtr pARK = ARKPTR(pScrn);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
-	IOADDRESS vgaIOBase = hwp->PIOOffset + hwp->IOBase;
+	unsigned long vgaIOBase = PIOOFFSET + hwp->IOBase;
 	int base;
 
 	base = ((y * pScrn->displayWidth + x) *
@@ -913,8 +919,8 @@ static void ARKWriteMode(ScrnInfoPtr pScrn, vgaRegPtr pVga, ARKRegPtr new)
 {
 	ARKPtr pARK = ARKPTR(pScrn);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
-	IOADDRESS isaIOBase = hwp->PIOOffset;
-	IOADDRESS vgaIOBase = isaIOBase + hwp->IOBase;
+	unsigned long isaIOBase = PIOOFFSET;
+	unsigned long vgaIOBase = isaIOBase + hwp->IOBase;
 
 	vgaHWProtect(pScrn, TRUE);
 
@@ -980,23 +986,23 @@ static void ARKWriteMode(ScrnInfoPtr pScrn, vgaRegPtr pVga, ARKRegPtr new)
 }
 
 
-static Bool ARKEnterVT(int scrnIndex, int flags)
+static Bool ARKEnterVT(VT_FUNC_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 
 	if (!ARKModeInit(pScrn, pScrn->currentMode))
 		return FALSE;
 
-	ARKAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+	ARKAdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
 
 	return TRUE;
 }
 
 
 
-static void ARKLeaveVT(int scrnIndex, int flags)
+static void ARKLeaveVT(VT_FUNC_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 	ARKPtr pARK = ARKPTR(pScrn);
 	ARKRegPtr old = &pARK->SavedRegs;
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
@@ -1022,24 +1028,28 @@ static Bool ARKMapMem(ScrnInfoPtr pScrn)
 					     pARK->PciTag, 0xb8000, 0x8000);
 
 	pARK->FBBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
-				     pARK->PciTag, pARK->FBAddress,
+				     pARK->PciTag, pARK->PciInfo->memBase[0],
 				     pScrn->videoRam * 1024);
 #else
 
-	pARK->MMIOBase = xf86MapDomainMemory(pScrn->scrnIndex, VIDMEM_MMIO,
-					     pARK->PciInfo, 0xb8000, 0x8000);
+	(void) pci_device_map_legacy(pARK->PciInfo, 0xb8000, 0x8000,
+				     PCI_DEV_MAP_FLAG_WRITABLE,
+				     &pARK->MMIOBase);
 
 	{
-		void** result = (void**)&pARK->FBBase;
+		void** result = &pARK->FBBase;
 		int err = pci_device_map_range(pARK->PciInfo,
-					       pARK->FBAddress,
+					       pARK->PciInfo->regions[0].base_addr,
 					       pScrn->videoRam * 1024,
 					       PCI_DEV_MAP_FLAG_WRITABLE |
 					       PCI_DEV_MAP_FLAG_WRITE_COMBINE,
 					       result);
 		
-		if (err) 
+		if (err) {
+			xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+			   "Cound not map framebuffer: %d\n", err);
 			return FALSE;
+		}
 	}
 #endif
 
@@ -1061,7 +1071,7 @@ static void ARKUnmapMem(ScrnInfoPtr pScrn)
 	vgaHWUnmapMem(pScrn);
 
 #ifndef XSERVER_LIBPCIACCESS
-	xf86UnMapVidMem(pScrn->scrnIndex, (pointer)pARK->FBBase,
+	xf86UnMapVidMem(pScrn->scrnIndex, pARK->FBBase,
 			pScrn->videoRam * 1024);
 #else
 	pci_device_unmap_range(pARK->PciInfo, pARK->FBBase, pScrn->videoRam * 1024);
@@ -1069,9 +1079,9 @@ static void ARKUnmapMem(ScrnInfoPtr pScrn)
 }
 
 
-Bool ARKCloseScreen(int scrnIndex, ScreenPtr pScreen)
+Bool ARKCloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	ARKPtr pARK = ARKPTR(pScrn);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
 
@@ -1087,7 +1097,7 @@ Bool ARKCloseScreen(int scrnIndex, ScreenPtr pScreen)
 
 	/* XXX Shouldn't XAADestroyInfoRec() be called? */
 
-	return (*pScreen->CloseScreen)(scrnIndex, pScreen);
+	return (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 }
 
 
@@ -1097,9 +1107,10 @@ Bool ARKSaveScreen(ScreenPtr pScreen, int mode)
 }
 
 
-Bool ARKSwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
+Bool ARKSwitchMode(SWITCH_MODE_ARGS_DECL)
 {
-	return ARKModeInit(xf86Screens[scrnIndex], mode);
+	SCRN_INFO_PTR(arg);
+	return ARKModeInit(pScrn, mode);
 }
 
 
@@ -1107,7 +1118,10 @@ static void ARKLoadPalette(ScrnInfoPtr pScrn, int numColors,
 			   int *indicies, LOCO *colors,
 			   VisualPtr pVisual)
 {
-	IOADDRESS isaIOBase = pScrn->domainIOBase;
+	unsigned long isaIOBase = 0;
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 12
+	isaIOBase += pScrn->domainIOBase;
+#endif
 	int i, index;
 
 	for (i=0; i<numColors; i++) {
@@ -1120,9 +1134,9 @@ static void ARKLoadPalette(ScrnInfoPtr pScrn, int numColors,
 }
 
 
-static void ARKFreeScreen(int scrnIndex, int flags)
+static void ARKFreeScreen(FREE_SCREEN_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 
 	vgaHWFreeHWRec(pScrn);
 
@@ -1130,7 +1144,7 @@ static void ARKFreeScreen(int scrnIndex, int flags)
 }
 
 
-static unsigned char get_daccomm(IOADDRESS isaIOBase)
+static unsigned char get_daccomm(unsigned long isaIOBase)
 {
 	unsigned char tmp;
 
@@ -1146,7 +1160,7 @@ static unsigned char get_daccomm(IOADDRESS isaIOBase)
 }
 
 
-static unsigned char set_daccom(IOADDRESS isaIOBase, unsigned char comm)
+static unsigned char set_daccom(unsigned long isaIOBase, unsigned char comm)
 {
 #if 0
 	outb(isaIOBase + 0x3c8, 0);
